local Cartographer3 = _G.Cartographer3
if not Cartographer3 then
	error("Cartographer3_Waypoints requires Cartographer3")
end

local L = Cartographer3.L

local Cartographer3_Waypoints = {}
Cartographer3.Waypoints = Cartographer3_Waypoints
_G.Cartographer3_Waypoints = Cartographer3_Waypoints -- add a global for easy checking from 3rd parties

local Cartographer3_Utils = Cartographer3.Utils

local db

local updateArrow

local waypoint_x, waypoint_y, waypoint_text, waypoint_metadata

function Cartographer3_Waypoints.SetWaypoint(zone, x, y, text, metadata)
	local ux, uy = Cartographer3_Utils.ConvertZoneCoordinateToUniverseCoordinate(zone, x, y)
	
	waypoint_x = ux
	waypoint_y = uy
	waypoint_text = text
	waypoint_metadata = metadata
	
	updateArrow(0, GetTime())
end
_G.SetWaypoint = Cartographer3_Waypoints.SetWaypoint

function Cartographer3_Waypoints.GetWaypoint()
	return waypoint_metadata
end
_G.GetWaypoint = Cartographer3_Waypoints.GetWaypoint

function Cartographer3_Waypoints.ClearWaypoint()
	waypoint_x = nil
	waypoint_y = nil
	waypoint_text = nil
	waypoint_metadata = nil
	
	updateArrow(0, GetTime())
end
_G.ClearWaypoint = Cartographer3_Waypoints.ClearWaypoint

do
	local button = CreateFrame("Button", "Cartographer3_Waypoints_Button", UIParent)
	button:SetWidth(100)
	button:SetHeight(55)
	button:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	button:EnableMouse(true)
	button:SetMovable(true)
	button:RegisterForDrag("LeftButton")
	button:SetScript("OnDragStart", function(this)
		this:StartMoving()
	end)
	button:SetScript("OnDragStop", function(this)
		this:StopMovingOrSizing()
	end)
	button:RegisterForClicks("RightButtonUp")
	button:Hide()
	button:SetClampedToScreen(true)
	local dropdownFrame
	local menuList
	local locked = false
	button:SetScript("OnClick", function(this)
		if not dropdownFrame then
			dropdownFrame = CreateFrame("Frame", this:GetName() .. "_Dropdown", this, "UIDropDownMenuTemplate")
			-- Clear waypoint
			-- Lock arrow
			menuList = {
				{
					text = L["Cartographer3 Waypoint Arrow"],
					isTitle = true,
				},
				{
					text = L["Clear waypoint"],
					func = Cartographer3_Waypoints.ClearWaypoint,
				},
				{
					text = L["Lock arrow position"],
					func = function()
						locked = not locked
						if locked then
							button:SetMovable(false)
							button:RegisterForDrag()
						else
							button:SetMovable(true)
							button:RegisterForDrag("LeftButton")
						end
						menuList[3].checked = locked
					end,
					checked = locked
				},
				{
					text = " ",
					isTitle = true,
				},
				{
					text = _G.CLOSE,
					func = function() end,
				}
			}
		end
		EasyMenu(menuList, dropdownFrame, "cursor", 0, 0, "MENU")
	end)
	
	local arrow = CreateFrame("PlayerModel", "Cartographer3_Waypoints_ArrowModel", button)
	arrow:SetModel([=[Interface\AddOns\Cartographer3\WaypointArrow.m2]=])
	arrow:SetModelScale(1.55)
	arrow:SetLight(1, 0, 0, 1, 0, 1, 0.7, 0.7, 0.7, 1, 0.8, 0.8, 0.64)
	arrow:SetPoint("CENTER")
	arrow:SetWidth(button:GetWidth())
	arrow:SetHeight(button:GetHeight())
	
	local text = arrow:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
	arrow.text = text
	text:SetPoint("TOP", arrow, "BOTTOM")
	
	local yardage = arrow:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	arrow.yardage = yardage
	yardage:SetPoint("TOP", text, "BOTTOM")
	
	local time = arrow:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	arrow.time = time
	time:SetPoint("TOP", yardage, "BOTTOM")
	
	local YARDS_PER_PIXEL = Cartographer3.Data.YARDS_PER_PIXEL
	
	local last_playerX, last_playerY
	local lastUpdate = 0
	function updateArrow(elapsed, currentTime)
		if not waypoint_x then
			button:Hide()
			return
		end
		
		local playerX, playerY = Cartographer3.Utils.GetUnitUniverseCoordinate("player")
		if not playerX then
			button:Hide()
			return
		end
		
		local destinationDeltaX, destinationDeltaY = waypoint_x - playerX, waypoint_y - playerY
		
		local angleToDestination = math.atan2(destinationDeltaY, destinationDeltaX)
		
		local playerRotation = Cartographer3.Utils.GetPlayerRotation()
		
		local arrowRotation = angleToDestination + playerRotation
		arrow:SetRotation(arrowRotation)
		
		local value = (((arrowRotation / math.pi) % 2) - 1)^2
		
		if value >= 0.5 then
			r, g, b = 2 - value*2, 1, 0
		else
			r, g, b = 1, value*2, 0
		end
		arrow:SetLight(1, 0, 0, 0.2, -0.8, 0.5, r, g, b, 0.8, r, g, b)
		
		arrow.text:SetText(waypoint_text)
		
		local yardsToDestination = (destinationDeltaX*destinationDeltaX + destinationDeltaY*destinationDeltaY)^0.5 * YARDS_PER_PIXEL
		
		if yardsToDestination < 5 then -- allow changing of yards
			Cartographer3_Waypoints.ClearWaypoint()
			return
		end
		
		local yardString = ("%.0f yd"):format(yardsToDestination)
		arrow.yardage:SetText(yardString)
		
		local diffTime = currentTime - lastUpdate
		if diffTime >= 1 then
			lastUpdate = currentTime
			if not last_playerX then
				last_playerX, last_playerY = playerX, playerY
				return
			end
			
			local movementX, movementY = playerX - last_playerX, playerY - last_playerY
			local movementAngle = math.atan2(movementY, movementX)
			local movementYardsPerSecond = (movementX*movementX + movementY*movementY)^0.5 * YARDS_PER_PIXEL / diffTime
			movementYardsPerSecond = math.floor(movementYardsPerSecond * 100 + 0.5) / 100
			
			local secondsToDestination = yardsToDestination / movementYardsPerSecond
			
			local adjustedSecondsToDestination = secondsToDestination / math.cos(movementAngle - angleToDestination)
			
			local timeString
			if adjustedSecondsToDestination < 0 or adjustedSecondsToDestination > 24*60*60 then
				timeString = '---'
			else
				if adjustedSecondsToDestination > 60*60 then
					timeString = ("%d:%02d:%02d"):format(adjustedSecondsToDestination / (60*60), (adjustedSecondsToDestination % (60*60)) / 60, adjustedSecondsToDestination % 60)
				else
					timeString = ("%d:%02d"):format(adjustedSecondsToDestination / 60, adjustedSecondsToDestination % 60)
				end
			end
		
			arrow.time:SetText(timeString)
			last_playerX, last_playerY = playerX, playerY
		end
		
		if not button:IsShown() then
			arrow:SetModel([=[Interface\AddOns\Cartographer3\WaypointArrow.m2]=])
			button:Show()
		end	
	end
	Cartographer3_Utils.AddTimer(updateArrow, true)
end

function Cartographer3_Waypoints.SetArrowSize(value)
	db.arrowSize = value
	Cartographer3_Waypoints_Button:SetScale(value)
	Cartographer3_Waypoints_ArrowModel:SetModelScale(1.55 / value)
end

function Cartographer3_Waypoints.OnInitialize()
	if not Cartographer3.db.Waypoints then
		Cartographer3.db.Waypoints = {}
	end
	db = Cartographer3.db.Waypoints
	if not db.arrowSize then
		db.arrowSize = 1
	end
	Cartographer3_Waypoints.SetArrowSize(db.arrowSize)
end

Cartographer3.MakeSuboptionsPanel(L["Waypoints"], function(self)
	local title = self:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetText(("Cartographer3 - %s"):format(L["Waypoints"]))
	title:SetJustifyH("LEFT")
	title:SetJustifyV("TOP")
	title:SetPoint("TOPLEFT", 16, -16)

	local subText = self:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	subText:SetText(L["These options allow you to configure Cartographer3's waypoints"]) -- TODO: better text
	subText:SetNonSpaceWrap(true)
	subText:SetJustifyH("LEFT")
	subText:SetJustifyV("TOP")
	subText:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
	subText:SetPoint("RIGHT", -32, 0)
	
	local function makeArrowSizeSlider()
		local arrowSizeSlider = CreateFrame("Slider", self:GetName() .. "_ArrowSizeSlider", self, "InterfaceOptionsSliderTemplate")
		self.controls[arrowSizeSlider] = true
		_G[arrowSizeSlider:GetName() .. "Text"]:SetText(L["Arrow Size"])
		arrowSizeSlider.tooltipText = L["The size of the waypoint arrow"]
		_G[arrowSizeSlider:GetName() .. "Text"]:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
		_G[arrowSizeSlider:GetName() .. "Low"]:SetText("25%")
		_G[arrowSizeSlider:GetName() .. "Low"]:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
		_G[arrowSizeSlider:GetName() .. "High"]:SetText("400%")
		_G[arrowSizeSlider:GetName() .. "High"]:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
		arrowSizeSlider.default = Cartographer3.Data.DEFAULT_ZOOM_TO_MINIMAP_TEXTURE
		arrowSizeSlider:SetMinMaxValues(0.25, 4)
		arrowSizeSlider.oldValue = db.arrowSize
		arrowSizeSlider:SetValue(arrowSizeSlider.oldValue)
		local oldSetValue = arrowSizeSlider.SetValue
		function arrowSizeSlider:SetValue(value)
			oldSetValue(self, value)
			self.value = value
			Cartographer3_Waypoints.SetArrowSize(value)
		end
		arrowSizeSlider:SetScript("OnValueChanged", function(self)
			self.value = self:GetValue()
			self:SetValue(self:GetValue())
		end)
		return arrowSizeSlider
	end
	
	local arrowSizeSlider = makeArrowSizeSlider()
	arrowSizeSlider:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", 0, -32)
end)
